(ns wanijo.infra.system-test
  (:require [clojure.test :refer :all]
            [neo4j-clj.core :as drv]
            [wanijo.infra.neo4j :as neo4j]
            [wanijo.infra.repl :as repl]
            [wanijo.schema.routes :as schema-routes]
            [wanijo.attribute.routes :as attr-routes]))

(defn single-result [cypher extractor]
  (extractor
   (first
    (neo4j/exec-query!
     (drv/create-query cypher) {}))))

(deftest allround-system-test
  (repl/setup-in-memory!)
  (def user
    (single-result "MATCH (u:user) RETURN u"
                   :u))

  (testing "create schema"
    (schema-routes/new! {:params {:name "test-schema"}
                         :session {:uuid (:uuid user)}})
    (def schema
      (single-result "MATCH (s:schema) RETURN s LIMIT 1" :s))
    (is (= "test-schema" (:name schema)))
    (is (= (:uuid user)
           (single-result "MATCH (s:schema)-[:created_by]->(u:user)
                           RETURN u.uuid AS uuid" :uuid)))
    (is (not= nil (:created_at schema))))

  (testing "create attribute"
    (attr-routes/new! {:params {:schema (:uuid schema)
                                :name "test-attr"
                                :type "string"
                                :required "1"}
                       :session {:uuid (:uuid user)}})
    (def attr
      (single-result "MATCH (a:attribute) RETURN a" :a))
    (is (= "string" (:type attr)))
    (is (= "test-attr" (:name attr)))
    (is (= 1 (:required attr)))
    (is (= (:uuid user)
           (single-result "MATCH (a:attribute)-[:created_by]->(u:user)
                           RETURN u.uuid AS uuid" :uuid)))
    (is (= (:uuid schema)
           (single-result "MATCH (a:attribute)-[:of]->(s:schema)
                           RETURN s.uuid AS uuid" :uuid)))))
